* {
    margin: 0;
    padding: 0;
    font-size: 100%;
}

body {
    font: 13px/1.4 "Helvetica Neue", Arial, Helvetica, sans-serif
}

header,
hgroup,
nav,
section,
article,
aside,
footer {
    display: block;
}