{% extends "news/base.html" %}
{% load news %}


{% block header %}{{block.super}} for {{year}}{% endblock %}


{% block content_primary %}

    {{block.super}}
    
    {% article_list article_list %}

{% endblock %}